@@ -14,42 +14,54 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
-from typing import List, NamedTuple, Optional, Tuple
+from typing import NamedTuple
 
 from marshmallow import Schema, ValidationError, fields, validate, validates_schema
 from marshmallow.utils import get_value
+from marshmallow_sqlalchemy import SQLAlchemySchema, auto_field
 
 from airflow.api_connexion.parameters import validate_istimezone
+from airflow.api_connexion.schemas.common_schema import JsonObjectField
 from airflow.api_connexion.schemas.enum_schemas import TaskInstanceStateField
 from airflow.api_connexion.schemas.sla_miss_schema import SlaMissSchema
 from airflow.models import SlaMiss, TaskInstance
+from airflow.utils.helpers import exactly_one
 from airflow.utils.state import State
 
 
-class TaskInstanceSchema(Schema):
+class TaskInstanceSchema(SQLAlchemySchema):
     """Task instance schema"""
 
-    task_id = fields.Str()
-    dag_id = fields.Str()
-    execution_date = fields.DateTime(validate=validate_istimezone)
-    start_date = fields.DateTime(validate=validate_istimezone)
-    end_date = fields.DateTime(validate=validate_istimezone)
-    duration = fields.Float()
+    class Meta:
+        """Meta"""
+
+        model = TaskInstance
+
+    task_id = auto_field()
+    dag_id = auto_field()
+    run_id = auto_field(data_key="dag_run_id")
+    map_index = auto_field()
+    execution_date = auto_field()
+    start_date = auto_field()
+    end_date = auto_field()
+    duration = auto_field()
     state = TaskInstanceStateField()
-    _try_number = fields.Int(data_key="try_number")
-    max_tries = fields.Int()
-    hostname = fields.Str()
-    unixname = fields.Str()
-    pool = fields.Str()
-    pool_slots = fields.Int()
-    queue = fields.Str()
-    priority_weight = fields.Int()
-    operator = fields.Str()
-    queued_dttm = fields.DateTime(data_key="queued_when")
-    pid = fields.Int()
-    executor_config = fields.Str()
+    _try_number = auto_field(data_key="try_number")
+    max_tries = auto_field()
+    hostname = auto_field()
+    unixname = auto_field()
+    pool = auto_field()
+    pool_slots = auto_field()
+    queue = auto_field()
+    priority_weight = auto_field()
+    operator = auto_field()
+    queued_dttm = auto_field(data_key="queued_when")
+    pid = auto_field()
+    executor_config = auto_field()
     sla_miss = fields.Nested(SlaMissSchema, dump_default=None)
+    rendered_fields = JsonObjectField(dump_default={})
 
     def get_attribute(self, obj, attr, default):
         if attr == "sla_miss":
@@ -58,13 +70,15 @@ class TaskInstanceSchema(Schema):
             # corresponding to the attr.
             slamiss_instance = {"sla_miss": obj[1]}
             return get_value(slamiss_instance, attr, default)
+        elif attr == "rendered_fields":
+            return get_value(obj[0], "rendered_task_instance_fields.rendered_fields", default)
         return get_value(obj[0], attr, default)
 
 
 class TaskInstanceCollection(NamedTuple):
     """List of task instances with metadata"""
 
-    task_instances: List[Tuple[TaskInstance, Optional[SlaMiss]]]
+    task_instances: list[tuple[TaskInstance, SlaMiss | None]]
     total_entries: int
 
 
@@ -106,6 +120,11 @@ class ClearTaskInstanceFormSchema(Schema):
     include_parentdag = fields.Boolean(load_default=False)
     reset_dag_runs = fields.Boolean(load_default=False)
     task_ids = fields.List(fields.String(), validate=validate.Length(min=1))
+    dag_run_id = fields.Str(load_default=None)
+    include_upstream = fields.Boolean(load_default=False)
+    include_downstream = fields.Boolean(load_default=False)
+    include_future = fields.Boolean(load_default=False)
+    include_past = fields.Boolean(load_default=False)
 
     @validates_schema
     def validate_form(self, data, **kwargs):
@@ -115,6 +134,12 @@ class ClearTaskInstanceFormSchema(Schema):
         if data["start_date"] and data["end_date"]:
             if data["start_date"] > data["end_date"]:
                 raise ValidationError("end_date is sooner than start_date")
+        if data["start_date"] and data["end_date"] and data["dag_run_id"]:
+            raise ValidationError("Exactly one of dag_run_id or (start_date and end_date) must be provided")
+        if data["start_date"] and data["dag_run_id"]:
+            raise ValidationError("Exactly one of dag_run_id or start_date must be provided")
+        if data["end_date"] and data["dag_run_id"]:
+            raise ValidationError("Exactly one of dag_run_id or end_date must be provided")
 
 
 class SetTaskInstanceStateFormSchema(Schema):
@@ -122,13 +147,20 @@ class SetTaskInstanceStateFormSchema(Schema):
 
     dry_run = fields.Boolean(dump_default=True)
     task_id = fields.Str(required=True)
-    execution_date = fields.DateTime(required=True, validate=validate_istimezone)
+    execution_date = fields.DateTime(validate=validate_istimezone)
+    dag_run_id = fields.Str()
     include_upstream = fields.Boolean(required=True)
     include_downstream = fields.Boolean(required=True)
     include_future = fields.Boolean(required=True)
     include_past = fields.Boolean(required=True)
     new_state = TaskInstanceStateField(required=True, validate=validate.OneOf([State.SUCCESS, State.FAILED]))
 
+    @validates_schema
+    def validate_form(self, data, **kwargs):
+        """Validates set task instance state form"""
+        if not exactly_one(data.get("execution_date"), data.get("dag_run_id")):
+            raise ValidationError("Exactly one of execution_date or dag_run_id must be provided")
+
 
 class TaskInstanceReferenceSchema(Schema):
     """Schema for the task instance reference schema"""
@@ -142,7 +174,7 @@ class TaskInstanceReferenceSchema(Schema):
 class TaskInstanceReferenceCollection(NamedTuple):
     """List of objects with metadata about taskinstance and dag_run_id"""
 
-    task_instances: List[Tuple[TaskInstance, str]]
+    task_instances: list[tuple[TaskInstance, str]]
 
 
 class TaskInstanceReferenceCollectionSchema(Schema):
