@@ -14,14 +14,15 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 import inspect
 from textwrap import dedent
-from typing import Callable, Optional, TypeVar
+from typing import Callable, Sequence
 
-from airflow.decorators.base import DecoratedOperator, task_decorator_factory
+from airflow.decorators.base import DecoratedOperator, TaskDecorator, task_decorator_factory
 from airflow.operators.python import PythonVirtualenvOperator
-from airflow.utils.python_virtualenv import remove_task_decorator
+from airflow.utils.decorators import remove_task_decorator
 
 
 class _PythonVirtualenvDecoratedOperator(DecoratedOperator, PythonVirtualenvOperator):
@@ -29,35 +30,36 @@ class _PythonVirtualenvDecoratedOperator(DecoratedOperator, PythonVirtualenvOper
     Wraps a Python callable and captures args/kwargs when called for execution.
 
     :param python_callable: A reference to an object that is callable
-    :type python_callable: python callable
     :param op_kwargs: a dictionary of keyword arguments that will get unpacked
         in your function (templated)
-    :type op_kwargs: dict
     :param op_args: a list of positional arguments that will get unpacked when
         calling your callable (templated)
-    :type op_args: list
     :param multiple_outputs: If set to True, the decorated function's return value will be unrolled to
         multiple XCom values. Dict will unroll to XCom values with its keys as XCom keys. Defaults to False.
-    :type multiple_outputs: bool
     """
 
-    template_fields = ('op_args', 'op_kwargs')
+    template_fields: Sequence[str] = ('op_args', 'op_kwargs')
     template_fields_renderers = {"op_args": "py", "op_kwargs": "py"}
 
     # since we won't mutate the arguments, we should just do the shallow copy
     # there are some cases we can't deepcopy the objects (e.g protobuf).
-    shallow_copy_attrs = ('python_callable',)
+    shallow_copy_attrs: Sequence[str] = ('python_callable',)
 
-    def __init__(
-        self,
-        **kwargs,
-    ) -> None:
+    custom_operator_name: str = '@task.virtualenv'
+
+    def __init__(self, *, python_callable, op_args, op_kwargs, **kwargs) -> None:
         kwargs_to_upstream = {
-            "python_callable": kwargs["python_callable"],
-            "op_args": kwargs["op_args"],
-            "op_kwargs": kwargs["op_kwargs"],
+            "python_callable": python_callable,
+            "op_args": op_args,
+            "op_kwargs": op_kwargs,
         }
-        super().__init__(kwargs_to_upstream=kwargs_to_upstream, **kwargs)
+        super().__init__(
+            kwargs_to_upstream=kwargs_to_upstream,
+            python_callable=python_callable,
+            op_args=op_args,
+            op_kwargs=op_kwargs,
+            **kwargs,
+        )
 
     def get_python_source(self):
         raw_source = inspect.getsource(self.python_callable)
@@ -66,35 +68,27 @@ class _PythonVirtualenvDecoratedOperator(DecoratedOperator, PythonVirtualenvOper
         return res
 
 
-T = TypeVar("T", bound=Callable)
+def virtualenv_task(
+    python_callable: Callable | None = None,
+    multiple_outputs: bool | None = None,
+    **kwargs,
+) -> TaskDecorator:
+    """Wraps a callable into an Airflow operator to run via a Python virtual environment.
 
+    Accepts kwargs for operator kwarg. Can be reused in a single DAG.
 
-class PythonVirtualenvDecoratorMixin:
-    """
-    Helper class for inheritance. This class is only used for the __init__.pyi so that IDEs
-    will autocomplete docker decorator functions
+    This function is only used only used during type checking or auto-completion.
 
     :meta private:
+
+    :param python_callable: Function to decorate
+    :param multiple_outputs: If set to True, the decorated function's return value will be unrolled to
+        multiple XCom values. Dict will unroll to XCom values with its keys as XCom keys.
+        Defaults to False.
     """
-
-    def virtualenv(
-        self, python_callable: Optional[Callable] = None, multiple_outputs: Optional[bool] = None, **kwargs
-    ):
-        """
-        Wraps a python function into an Airflow operator to run via a Python virtual environment.
-
-        Accepts kwargs for operator kwarg. Can be reused in a single DAG.
-
-        :param python_callable: Function to decorate
-        :type python_callable: Optional[Callable]
-        :param multiple_outputs: If set to True, the decorated function's return value will be unrolled to
-            multiple XCom values. Dict will unroll to XCom values with its keys as XCom keys.
-            Defaults to False.
-        :type multiple_outputs: bool
-        """
-        return task_decorator_factory(
-            python_callable=python_callable,
-            multiple_outputs=multiple_outputs,
-            decorated_operator_class=_PythonVirtualenvDecoratedOperator,
-            **kwargs,
-        )
+    return task_decorator_factory(
+        python_callable=python_callable,
+        multiple_outputs=multiple_outputs,
+        decorated_operator_class=_PythonVirtualenvDecoratedOperator,
+        **kwargs,
+    )
