@@ -18,27 +18,30 @@
 """
 Example LatestOnlyOperator and TriggerRule interactions
 """
+from __future__ import annotations
 
 # [START example]
-import datetime as dt
+import datetime
+
+import pendulum
 
 from airflow import DAG
-from airflow.operators.dummy import DummyOperator
+from airflow.operators.empty import EmptyOperator
 from airflow.operators.latest_only import LatestOnlyOperator
 from airflow.utils.trigger_rule import TriggerRule
 
 with DAG(
     dag_id='latest_only_with_trigger',
-    schedule_interval=dt.timedelta(hours=4),
-    start_date=dt.datetime(2021, 1, 1),
+    schedule=datetime.timedelta(hours=4),
+    start_date=pendulum.datetime(2021, 1, 1, tz="UTC"),
     catchup=False,
     tags=['example3'],
 ) as dag:
     latest_only = LatestOnlyOperator(task_id='latest_only')
-    task1 = DummyOperator(task_id='task1')
-    task2 = DummyOperator(task_id='task2')
-    task3 = DummyOperator(task_id='task3')
-    task4 = DummyOperator(task_id='task4', trigger_rule=TriggerRule.ALL_DONE)
+    task1 = EmptyOperator(task_id='task1')
+    task2 = EmptyOperator(task_id='task2')
+    task3 = EmptyOperator(task_id='task3')
+    task4 = EmptyOperator(task_id='task4', trigger_rule=TriggerRule.ALL_DONE)
 
     latest_only >> task1 >> [task3, task4]
     task2 >> [task3, task4]
