@@ -15,14 +15,16 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """Example DAG demonstrating the usage of the ShortCircuitOperator."""
+from __future__ import annotations
+
 import pendulum
 
 from airflow import DAG
 from airflow.models.baseoperator import chain
-from airflow.operators.dummy import DummyOperator
+from airflow.operators.empty import EmptyOperator
 from airflow.operators.python import ShortCircuitOperator
+from airflow.utils.trigger_rule import TriggerRule
 
 with DAG(
     dag_id='example_short_circuit_operator',
@@ -40,8 +42,20 @@ with DAG(
         python_callable=lambda: False,
     )
 
-    ds_true = [DummyOperator(task_id='true_' + str(i)) for i in [1, 2]]
-    ds_false = [DummyOperator(task_id='false_' + str(i)) for i in [1, 2]]
+    ds_true = [EmptyOperator(task_id='true_' + str(i)) for i in [1, 2]]
+    ds_false = [EmptyOperator(task_id='false_' + str(i)) for i in [1, 2]]
 
     chain(cond_true, *ds_true)
     chain(cond_false, *ds_false)
+
+    [task_1, task_2, task_3, task_4, task_5, task_6] = [
+        EmptyOperator(task_id=f"task_{i}") for i in range(1, 7)
+    ]
+
+    task_7 = EmptyOperator(task_id="task_7", trigger_rule=TriggerRule.ALL_DONE)
+
+    short_circuit = ShortCircuitOperator(
+        task_id="short_circuit", ignore_downstream_trigger_rules=False, python_callable=lambda: False
+    )
+
+    chain(task_1, [task_2, short_circuit], [task_3, task_4], [task_5, task_6], task_7)
