@@ -15,12 +15,13 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """
 Example usage of the TriggerDagRunOperator. This example holds 2 DAGs:
 1. 1st DAG (example_trigger_controller_dag) holds a TriggerDagRunOperator, which will trigger the 2nd DAG
 2. 2nd DAG (example_trigger_target_dag) which will be triggered by the TriggerDagRunOperator in the 1st DAG
 """
+from __future__ import annotations
+
 import pendulum
 
 from airflow import DAG
@@ -34,7 +35,6 @@ def run_this_func(dag_run=None):
     Print the payload "message" passed to the DagRun conf attribute.
 
     :param dag_run: The DagRun object
-    :type dag_run: DagRun
     """
     print(f"Remotely received value of {dag_run.conf.get('message')} for key=message")
 
@@ -43,7 +43,7 @@ with DAG(
     dag_id="example_trigger_target_dag",
     start_date=pendulum.datetime(2021, 1, 1, tz="UTC"),
     catchup=False,
-    schedule_interval=None,
+    schedule=None,
     tags=['example'],
 ) as dag:
     run_this = run_this_func()
