@@ -15,23 +15,29 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
+import contextlib
 from logging.config import fileConfig
 
 from alembic import context
 
 from airflow import models, settings
+from airflow.utils.db import compare_server_default, compare_type
 
 
 def include_object(_, name, type_, *args):
     """Filter objects for autogenerating revisions"""
-    # Ignore _anything_ to do with Flask AppBuilder's tables
-    if type_ == "table" and name.startswith("ab_"):
+    # Ignore _anything_ to do with Celery, or FlaskSession's tables
+    if type_ == "table" and (name.startswith("celery_") or name == "session"):
         return False
     else:
         return True
 
 
+# Make sure everything is imported so that alembic can find it all
+models.import_all_models()
+
 # this is the Alembic Config object, which provides
 # access to the values within the .ini file in use.
 config = context.config
@@ -51,8 +57,6 @@ target_metadata = models.base.Base.metadata
 # my_important_option = config.get_main_option("my_important_option")
 # ... etc.
 
-COMPARE_TYPE = False
-
 
 def run_migrations_offline():
     """Run migrations in 'offline' mode.
@@ -70,7 +74,8 @@ def run_migrations_offline():
         url=settings.SQL_ALCHEMY_CONN,
         target_metadata=target_metadata,
         literal_binds=True,
-        compare_type=COMPARE_TYPE,
+        compare_type=compare_type,
+        compare_server_default=compare_server_default,
         render_as_batch=True,
     )
 
@@ -85,14 +90,18 @@ def run_migrations_online():
     and associate a connection with the context.
 
     """
-    connectable = settings.engine
+    with contextlib.ExitStack() as stack:
+        connection = config.attributes.get('connection', None)
+
+        if not connection:
+            connection = stack.push(settings.engine.connect())
 
-    with connectable.connect() as connection:
         context.configure(
             connection=connection,
             transaction_per_migration=True,
             target_metadata=target_metadata,
-            compare_type=COMPARE_TYPE,
+            compare_type=compare_type,
+            compare_server_default=compare_server_default,
             include_object=include_object,
             render_as_batch=True,
         )
