@@ -15,11 +15,14 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
+
 import datetime
 
 from airflow.sensors.base import BaseSensorOperator
 from airflow.triggers.temporal import DateTimeTrigger
 from airflow.utils import timezone
+from airflow.utils.context import Context
 
 
 class TimeSensor(BaseSensorOperator):
@@ -27,14 +30,13 @@ class TimeSensor(BaseSensorOperator):
     Waits until the specified time of the day.
 
     :param target_time: time after which the job succeeds
-    :type target_time: datetime.time
     """
 
     def __init__(self, *, target_time, **kwargs):
         super().__init__(**kwargs)
         self.target_time = target_time
 
-    def poke(self, context):
+    def poke(self, context: Context):
         self.log.info('Checking if the time (%s) has come', self.target_time)
         return timezone.make_naive(timezone.utcnow(), self.dag.timezone).time() > self.target_time
 
@@ -45,23 +47,24 @@ class TimeSensorAsync(BaseSensorOperator):
     it is waiting.
 
     :param target_time: time after which the job succeeds
-    :type target_time: datetime.time
     """
 
     def __init__(self, *, target_time, **kwargs):
         super().__init__(**kwargs)
         self.target_time = target_time
 
-        self.target_datetime = timezone.coerce_datetime(
+        aware_time = timezone.coerce_datetime(
             datetime.datetime.combine(datetime.datetime.today(), self.target_time)
         )
 
-    def execute(self, context):
+        self.target_datetime = timezone.convert_to_utc(aware_time)
+
+    def execute(self, context: Context):
         self.defer(
             trigger=DateTimeTrigger(moment=self.target_datetime),
             method_name="execute_complete",
         )
 
-    def execute_complete(self, context, event=None):  # pylint: disable=unused-argument
+    def execute_complete(self, context, event=None):
         """Callback for when the trigger fires - returns immediately."""
         return None
