@@ -15,8 +15,8 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-
 """Task runner for cgroup to run Airflow task"""
+from __future__ import annotations
 
 import datetime
 import os
@@ -133,9 +133,7 @@ class CgroupTaskRunner(BaseTaskRunner):
             return
 
         # Create a unique cgroup name
-        cgroup_name = "airflow/{}/{}".format(
-            datetime.datetime.utcnow().strftime("%Y-%m-%d"), str(uuid.uuid4())
-        )
+        cgroup_name = f"airflow/{datetime.datetime.utcnow().strftime('%Y-%m-%d')}/{str(uuid.uuid4())}"
 
         self.mem_cgroup_name = f"memory/{cgroup_name}"
         self.cpu_cgroup_name = f"cpu/{cgroup_name}"
@@ -148,11 +146,11 @@ class CgroupTaskRunner(BaseTaskRunner):
         self._mem_mb_limit = resources.ram.qty
 
         # Create the memory cgroup
-        mem_cgroup_node = self._create_cgroup(self.mem_cgroup_name)
+        self.mem_cgroup_node = self._create_cgroup(self.mem_cgroup_name)
         self._created_mem_cgroup = True
         if self._mem_mb_limit > 0:
             self.log.debug("Setting %s with %s MB of memory", self.mem_cgroup_name, self._mem_mb_limit)
-            mem_cgroup_node.controller.limit_in_bytes = self._mem_mb_limit * 1024 * 1024
+            self.mem_cgroup_node.controller.limit_in_bytes = self._mem_mb_limit * 1024 * 1024
 
         # Create the CPU cgroup
         cpu_cgroup_node = self._create_cgroup(self.cpu_cgroup_name)
@@ -165,7 +163,7 @@ class CgroupTaskRunner(BaseTaskRunner):
         self.log.debug("Starting task process with cgroups cpu,memory: %s", cgroup_name)
         self.process = self.run_command(['cgexec', '-g', f'cpu,memory:{cgroup_name}'])
 
-    def return_code(self):
+    def return_code(self, timeout: int = 0) -> int | None:
         return_code = self.process.poll()
         # TODO(plypaul) Monitoring the control file in the cgroup fs is better than
         # checking the return code here. The PR to use this is here:
@@ -187,11 +185,32 @@ class CgroupTaskRunner(BaseTaskRunner):
         if self.process and psutil.pid_exists(self.process.pid):
             reap_process_group(self.process.pid, self.log)
 
+    def _log_memory_usage(self, mem_cgroup_node):
+        def byte_to_gb(num_bytes, precision=2):
+            return round(num_bytes / (1024 * 1024 * 1024), precision)
+
+        with open(mem_cgroup_node.full_path + '/memory.max_usage_in_bytes') as f:
+            max_usage_in_bytes = int(f.read().strip())
+
+        used_gb = byte_to_gb(max_usage_in_bytes)
+        limit_gb = byte_to_gb(mem_cgroup_node.controller.limit_in_bytes)
+
+        self.log.info(
+            "Memory max usage of the task is %s GB, while the memory limit is %s GB", used_gb, limit_gb
+        )
+
+        if max_usage_in_bytes >= mem_cgroup_node.controller.limit_in_bytes:
+            self.log.info(
+                "This task has reached the memory limit allocated by Airflow worker. "
+                "If it failed, try to optimize the task or reserve more memory."
+            )
+
     def on_finish(self):
         # Let the OOM watcher thread know we're done to avoid false OOM alarms
         self._finished_running = True
         # Clean up the cgroups
         if self._created_mem_cgroup:
+            self._log_memory_usage(self.mem_cgroup_node)
             self._delete_cgroup(self.mem_cgroup_name)
         if self._created_cpu_cgroup:
             self._delete_cgroup(self.cpu_cgroup_name)
