@@ -29,6 +29,7 @@ from airflow.configuration import AirflowConfigException, conf
 from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.utils.context import Context
 from airflow.utils.helpers import parse_template_string, render_template_to_string
+from airflow.utils.log.logging_mixin import DISABLE_PROPOGATE
 from airflow.utils.log.non_caching_file_handler import NonCachingFileHandler
 from airflow.utils.session import create_session
 
@@ -72,6 +73,8 @@ class FileTaskHandler(logging.Handler):
             self.handler.setFormatter(self.formatter)
         self.handler.setLevel(self.level)
 
+        return DISABLE_PROPOGATE
+
     def emit(self, record):
         if self.handler:
             self.handler.emit(record)
