@@ -14,6 +14,7 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
+from __future__ import annotations
 
 import logging
 import warnings
@@ -24,8 +25,8 @@ from flask import Flask, request
 
 from airflow.api_connexion.exceptions import common_error_handler
 from airflow.configuration import conf
+from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.security import permissions
-from airflow.www.views import lazy_add_provider_discovered_options_to_connection_form
 
 log = logging.getLogger(__name__)
 
@@ -42,9 +43,14 @@ def init_flash_views(app):
 
 def init_appbuilder_views(app):
     """Initialize Web UI views"""
-    appbuilder = app.appbuilder
+    from airflow.models import import_all_models
+
+    import_all_models()
+
     from airflow.www import views
 
+    appbuilder = app.appbuilder
+
     # Remove the session from scoped_session registry to avoid
     # reusing a session with a disconnected connection
     appbuilder.session.remove()
@@ -143,6 +149,8 @@ def init_plugins(app):
 
 def init_connection_form():
     """Initializes connection form"""
+    from airflow.www.views import lazy_add_provider_discovered_options_to_connection_form
+
     lazy_add_provider_discovered_options_to_connection_form()
 
 
@@ -180,8 +188,7 @@ def init_api_connexion(app: Flask) -> None:
     from airflow.www import views
 
     @app.errorhandler(404)
-    @app.errorhandler(405)
-    def _handle_api_error(ex):
+    def _handle_api_not_found(ex):
         if request.path.startswith(base_path):
             # 404 errors are never handled on the blueprint level
             # unless raised from a view func so actual 404 errors,
@@ -191,6 +198,13 @@ def init_api_connexion(app: Flask) -> None:
         else:
             return views.not_found(ex)
 
+    @app.errorhandler(405)
+    def _handle_method_not_allowed(ex):
+        if request.path.startswith(base_path):
+            return common_error_handler(ex)
+        else:
+            return views.method_not_allowed(ex)
+
     spec_dir = path.join(ROOT_APP_DIR, 'api_connexion', 'openapi')
     connexion_app = App(__name__, specification_dir=spec_dir, skip_error_handlers=True)
     connexion_app.app = app
@@ -214,7 +228,7 @@ def init_api_experimental(app):
         "The experimental REST API is deprecated. Please migrate to the stable REST API. "
         "Please note that the experimental API do not have access control. "
         "The authenticated user has full access.",
-        DeprecationWarning,
+        RemovedInAirflow3Warning,
     )
     app.register_blueprint(endpoints.api_experimental, url_prefix='/api/experimental')
     app.extensions['csrf'].exempt(endpoints.api_experimental)
